/*
 * Copyright 2012-2013 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.ext;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Binds the specified {@link org.callbackparams.wrap.legacy.Wrapping Wrapping}
 * implementations to the annotated annotation type.
 *
 * It is also possible for the test developer to annotate the enum-constant
 * parameter-values (or classes of parameter values) directly but the
 * recommended pattern is to annotate other annotation-types.
 *
 * An sandbox-example, on how this annotation works, is
 * {@link org.callbackparams.sandbox.Property.Value}.
 *
 * @author Henrik Kaipe
 */
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Documented
@Target({ElementType.ANNOTATION_TYPE, ElementType.FIELD, ElementType.TYPE})
public @interface WrappingCallbackImpls {
    Class<? extends org.callbackparams.wrap.legacy.Wrapping>[] value();
}
