/*
 * Copyright 2012-2013 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.internal.template;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import org.callbackparams.support.ClassWrapper;
import org.callbackparams.support.JdkVersion;

/**
 * @author Henrik Kaipe
 */
public abstract class AnnotationSpecifiedCallbacks {

    public static final class DummyJdk14Impl
    extends AnnotationSpecifiedCallbacks {

        /* Dummy constructors ... */
        public DummyJdk14Impl(Method m, int i) {}
        public DummyJdk14Impl(Field f) {}

        /**
         * This functionality is unsupported for JDK-1.4, so the
         * method simply returns null.
         */
        protected Object asCallback(Object callbackValue) { return null; }
    }

    private final static ClassWrapper implementationClass =
            JdkVersion.supportsAnnotations() ? JdkVersion
            .compatibleImplementationOf(AnnotationSpecifiedCallbacks.class)
            : new ClassWrapper(DummyJdk14Impl.class);

    static AnnotationSpecifiedCallbacks forCallbackRef(Method m, int paramIndex) {
        return (AnnotationSpecifiedCallbacks) implementationClass
                .newInstance(new Object[] {m, new Integer(paramIndex)});
    }

    static AnnotationSpecifiedCallbacks forCallbackRef(Field f) {
        return (AnnotationSpecifiedCallbacks)
                implementationClass.newInstance(f);
    }

    abstract protected Object asCallback(Object callbackValue);
}
